@@ -24,7 +24,6 @@
  * 2002-05-19 Randy Dunlap <rddunlap@osdl.org>
  * - Modify for 'kerneltop'
  * - TBD:
- * . maybe add read_all_profile_and_clear() in kernel [i.e., atomic]
  * . make program know how many lines are on the console (instead
  *   of being told);
  * . add system/user/idle time per reporting period;
@@ -46,11 +45,20 @@
  * . don't reset /proc/profile; just do incremental/interval diffs of it;
  *   (from Bill Irwin <wli@holomorphy.com>);
  * . move total_ticks to a heading line, don't waste an entire line for it;
- *   (from cef-lkml@optusnet.com.au);
+ *   (from Stuart Young <cef-lkml@optusnet.com.au>);
  * . put "address  function ....." heading line in reverse video;
  * . add get_unames() to save uname info;
  * . if mapfile name is not specified (-m) and /boot/System.map is not
  *   found, try to open /boot/System.map-`uname -r`;
+ * 2004-05-27: Version 0.9:
+ * . correct getopt's optstring (remove 'n'; add 'r');
+ *   noticed by Stuart Young;
+ * . if a mapfile name is not specified, try to open in this order:
+ *   /boot/System.map-`uname -r`, /boot/System.map, and
+ *   /lib/modules/`uname -r`/System.map;
+ * . add man page (from Stuart Young);
+ * . remove ioctl's from keyboard I/O routine; not needed/used;
+ *   keyboard handling still needs some work;
  */
 
 #include <errno.h>
@@ -74,7 +82,7 @@
 #endif
 #include "vidsupport.h"
 
-#define VERSION "0.8"
+#define VERSION "0.9"
 
 // video output rows/lines & columns
 #define ROW_TITLE1	1
@@ -84,7 +92,7 @@
 #define ROW_DATA	4	// first row of function/usage data
 #define ROW_MESSAGE	1
 
-#define S_LEN		128
+#define S_LEN		1024	// fix debian bug #607309
 #define OUTPUT_SIZE	512
 
 #define DEF_NUMLINES	20		/* lines */
@@ -117,7 +125,7 @@ static char *prgname;
 /* These are the defaults */
 static char defaultmap[] = "/boot/System.map";
 static char defaultpro[] = "/proc/profile";
-static char optstring[] = "m:np:s:l:t:uV";
+static char optstring[] = "m:p:rs:l:t:uV";
 
 // some command-line options:
 int console_lines = DEF_NUMLINES;
@@ -173,7 +181,7 @@ static void
 usage(void) {
 	fprintf(stderr,
 		  _("%s: usage: \"%s [options]\n"
-		  "\t -m <mapfile>  (default = \"%s\")\n"
+		  "\t -m <mapfile>  (default = \"%s\"[-`uname -r`])\n"
 		  "\t -p <pro-file> (default = \"%s\")\n"
 		  "\t -l <lines>    set number of lines to print (def. = %d; max = %d)\n"
 		  "\t -s <seconds>  set sleep time in seconds (def. = 1)\n"
@@ -283,7 +291,7 @@ static void get_unames (char *uname_all, char *uname_release)
 			errno, strerror (errno));
 		exit (1);
 	}
-	memset (uname_all, 0, sizeof(uname_all));
+	memset (uname_all, 0, sizeof(*uname_all));
 	fread (uname_all, OUTPUT_SIZE - 1, 1, pin);
 	pclose (pin);
 
@@ -293,7 +301,7 @@ static void get_unames (char *uname_all, char *uname_release)
 			errno, strerror (errno));
 		exit (1);
 	}
-	memset (uname_release, 0, sizeof(uname_release));
+	memset (uname_release, 0, sizeof(*uname_release));
 	fread (uname_release, OUTPUT_SIZE - 1, 1, pin);
 	pclose (pin);
 	if (uname_release [strlen (uname_release) - 1] == '\n')
@@ -393,22 +401,24 @@ void helper (void)
 #include <string.h>
 #include <termios.h>
 #include <unistd.h>
-#include <sys/ioctl.h>
+///#include <sys/ioctl.h>
 #include <linux/kd.h>
 
 int keypressed (int keyboard_fd) {
 	char		one_key;
 	struct termios	termio_backup;
 	struct termios	termio_new;
+	ssize_t		actual;
+#ifdef RAW_MODE
 	int		old_mode;
 	int		meta_mode;
 	int		temp = K_RAW;
-	ssize_t		actual;
+#endif
 
 	tcgetattr(keyboard_fd, &termio_backup);
 	memcpy(&termio_new, &termio_backup, sizeof(struct termios));
-	ioctl(keyboard_fd, KDGKBMODE, &old_mode);
-	ioctl(keyboard_fd, KDGKBMETA, &meta_mode);
+///	ioctl(keyboard_fd, KDGKBMODE, &old_mode);
+///	ioctl(keyboard_fd, KDGKBMETA, &meta_mode);
 
 	termio_new.c_lflag      &= ~(ICANON|ECHO|ISIG);
 	termio_new.c_iflag      = 0;
@@ -417,8 +427,8 @@ int keypressed (int keyboard_fd) {
 	termio_new.c_cc[VMIN]   = 0;
 	termio_new.c_cc[VTIME]  = 0;
 	tcsetattr(keyboard_fd, TCSANOW, &termio_new);
-	ioctl(keyboard_fd, KDSKBMODE, temp);
-	ioctl(keyboard_fd, KDSKBMETA, temp);
+///	ioctl(keyboard_fd, KDSKBMODE, temp);
+///	ioctl(keyboard_fd, KDSKBMETA, temp);
 	///tcflush(keyboard_fd, TCIOFLUSH);
 
 	actual = read (keyboard_fd, &one_key, 1);
@@ -429,8 +439,8 @@ int keypressed (int keyboard_fd) {
 		ungetc (one_key, stdin);
 #endif
 
-	ioctl(keyboard_fd, KDSKBMODE, old_mode);
-	ioctl(keyboard_fd, KDSKBMETA, meta_mode);
+///	ioctl(keyboard_fd, KDSKBMODE, old_mode);
+///	ioctl(keyboard_fd, KDSKBMETA, meta_mode);
 	tcsetattr(keyboard_fd, TCSANOW, &termio_backup);
 	///tcflush(keyboard_fd, TCIOFLUSH);
 
@@ -574,20 +584,39 @@ static void mapfile_read (const char *mapFile, int opt_mapfile)
 	char begin_text[] = "_stext";	// want all T/t between begin/end text
 	char end_text[] = "_etext";	// this one is Absolute.
 
-	if (!(mapf = myopen (mapFile, "r", &popenMap))) {
-		// if !opt_mapfile, try default + uname_release;
-		if (!opt_mapfile) {
-			strcpy (fn_name, defaultmap);
-			strcat (fn_name, "-");
-			strcat (fn_name, uname_release);
-			if ((mapf = myopen (fn_name, "r", &popenMap)))
-				goto file_ok;
-		}
-		fprintf (stderr, "%s: %s: %s\n",
-				prgname, mapFile, strerror (errno));
-		exit (1);
-	}
-file_ok:
+	// try specified/default name first.
+	if ((mapf = myopen (mapFile, "r", &popenMap)))
+		goto open_ok;
+	if (opt_mapfile)	// then don't try the defaults;
+		goto open_err;
+
+	// open order for default mapfile name is:
+	// (i.e., for filename not specified):
+	// /boot/System.map-`uname -r`, /boot/System.map,
+	// /lib/modules/`uname -r`/System.map (& that's all);
+
+	// try default + uname_release;
+	strcpy (fn_name, defaultmap);
+	strcat (fn_name, "-");
+	strcat (fn_name, uname_release);
+	if ((mapf = myopen (fn_name, "r", &popenMap)))
+		goto open_ok;
+	// try default without uname_release;
+	if ((mapf = myopen (defaultmap, "r", &popenMap)))
+		goto open_ok;
+	// try /lib/modules/uname_release/System.map;
+	strcpy (fn_name, "/lib/modules/");
+	strcat (fn_name, uname_release);
+	strcat (fn_name, "/System.map");
+	if ((mapf = myopen (fn_name, "r", &popenMap)))
+		goto open_ok;
+
+open_err:
+	fprintf (stderr, "%s: %s: %s\n",
+			prgname, mapFile, strerror (errno));
+	exit (1);
+
+open_ok:
 	// first find <begin_text>
 	while (fgets (mapline, S_LEN, mapf)) {
 		maplineno++;
@@ -774,7 +803,6 @@ main (int argc, char **argv) {
 	int ix, mx, fnx, slpx;
 	unsigned long adr;
 	int valid;
-	int first = 1;
 	char ch;
 
 #ifdef NLS
@@ -804,7 +832,7 @@ main (int argc, char **argv) {
 			  if (sleep_seconds <= 0)
 				  sleep_seconds = 1;
 			  break;
-		case 'n': optReverse++; break;
+		case 'r': optReverse++; break;
 		case 't': threshold = strtol (optarg, NULL, 0);
 			  if (threshold < 1)
 				  threshold = 1;
@@ -844,7 +872,6 @@ main (int argc, char **argv) {
 	vid_curpos (ROW_SAMPLING_RANGE, 1);
 	printf (_("Sampling_step: %i | Address range: 0x%lx - 0x%lx\n"),
 		step, adr0, adrz);
-	first = 0;
 
 	time (&timer);
 	dttm = localtime (&timer);
